from PySide2.QtCore import *
from PySide2.QtWidgets import *
from PySide2.QtGui import *


class VariantPlugin(QWidget):
    def __init__(self):
        super().__init__()

    def set_variant(self, variant):
        raise NotImplemented()
